@@ -414,6 +414,7 @@ NOEXPORT void client_run(CLI *c) {
 #endif /* __GNUC__ */
 
 NOEXPORT void client_try(CLI *c) {
+    c->flag.redirect=0;
     local_start(c);
     if(c->opt->protocol_early)
         c->opt->protocol_early(c);
@@ -424,11 +425,11 @@ NOEXPORT void client_try(CLI *c) {
         ssl_start(c);
     } else {
         ssl_start(c);
-        if(c->opt->protocol_middle)
+        if(c->opt->protocol_middle && !c->flag.redirect)
             c->opt->protocol_middle(c);
         remote_start(c);
     }
-    if(c->opt->protocol_late)
+    if(c->opt->protocol_late && !c->flag.redirect)
         c->opt->protocol_late(c);
     transfer(c);
 }
@@ -515,7 +516,7 @@ NOEXPORT void remote_start(CLI *c) {
         c->bind_addr=NULL; /* don't bind */
 
     /* setup c->remote_fd, now */
-    if(c->opt->exec_name && !c->opt->connect_addr.names && !redirect(c))
+    if(c->opt->exec_name && !c->opt->connect_addr.names && !c->flag.redirect)
         c->remote_fd.fd=connect_local(c); /* not for exec+connect targets */
     else
         c->remote_fd.fd=connect_remote(c);
@@ -669,7 +670,8 @@ NOEXPORT void ssl_start(CLI *c) {
             }
         }
         if(SSL_session_reused(c->ssl)) {
-            print_session_id(sess); /* otherwise printed from sess_new_cb() */
+            /* otherwise printed from sess_new_cb() */
+            print_session_id("Session id", sess);
         } else { /* a new session was negotiated */
             /* SSL_SESS_CACHE_NO_INTERNAL_STORE prevented automatic caching */
             if(!c->opt->option.client)
@@ -680,25 +682,25 @@ NOEXPORT void ssl_start(CLI *c) {
         s_log(LOG_ERR, "No session available for redirection");
         throw_exception(c, 1);
     }
+    c->flag.redirect=(unsigned)redirect(c)&1;
 }
 
 NOEXPORT void session_cache_retrieve(CLI *c) {
-    SSL_SESSION *sess;
+    SSL_SESSION *sess=NULL;
 
     CRYPTO_THREAD_read_lock(stunnel_locks[LOCK_SESSION]);
-    if(c->opt->option.delayed_lookup) {
-        sess=c->opt->session;
-    } else { /* per-destination client cache */
-        if(c->opt->connect_session) {
-            sess=c->opt->connect_session[c->idx];
-        } else {
-            s_log(LOG_ERR, "INTERNAL ERROR: Uninitialized client session cache (retrieve)");
-            sess=NULL;
-        }
-    }
+    if(c->opt->connect_session) /* per-destination client session */
+        sess=c->opt->connect_session[c->idx];
+    if(!sess) /* either no per-destination cache or no session cached */
+        sess=c->opt->session; /* fallback session */
     if(sess)
         SSL_set_session(c->ssl, sess);
     CRYPTO_THREAD_unlock(stunnel_locks[LOCK_SESSION]);
+
+    if(sess)
+        print_session_id("Attempting to resume", sess);
+    else
+        s_log(LOG_DEBUG, "No previous session to resume");
 }
 
 #if OPENSSL_VERSION_NUMBER >= 0x10101000L
@@ -1663,7 +1665,7 @@ NOEXPORT unsigned idx_cache_retrieve(CLI *c) {
 }
 
 NOEXPORT void connect_setup(CLI *c) {
-    if(redirect(c)) { /* process "redirect" first */
+    if(c->flag.redirect) { /* process "redirect" first */
         s_log(LOG_NOTICE, "Redirecting connection");
         /* c->connect_addr.addr may be allocated in protocol negotiations */
         str_free(c->connect_addr.addr);
