@@ -109,6 +109,9 @@ NOEXPORT int ssl_tlsext_ticket_key_cb(SSL *, unsigned char *,
 NOEXPORT int sess_new_cb(SSL *, SSL_SESSION *);
 NOEXPORT void new_chain(CLI *);
 NOEXPORT void session_cache_save(CLI *, SSL_SESSION *);
+#if OPENSSL_VERSION_NUMBER<0x10101000L
+NOEXPORT SSL_SESSION *SSL_SESSION_dup(SSL_SESSION *);
+#endif
 NOEXPORT SSL_SESSION *sess_get_cb(SSL *,
 #if OPENSSL_VERSION_NUMBER>=0x10100000L
     const
@@ -128,8 +131,10 @@ NOEXPORT void cache_transfer(SSL_CTX *, const u_char, const long,
 /* info callbacks */
 NOEXPORT void info_callback(const SSL *, int, int);
 
-NOEXPORT void sslerror_queue(void);
-NOEXPORT void sslerror_log(unsigned long, const char *, int, const char *);
+#ifndef OPENSSL_NO_TLS1_3
+NOEXPORT char *compare_cipher_lists(STACK_OF(SSL_CIPHER) *, STACK_OF(SSL_CIPHER) *);
+NOEXPORT char *get_tls13_cipher_list(STACK_OF(SSL_CIPHER) *);
+#endif /* TLS 1.3 */
 
 /**************************************** initialize section->ctx */
 
@@ -216,11 +221,24 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
 #ifndef OPENSSL_NO_TLS1_3
     /* ciphersuites */
     if(section->ciphersuites) {
-        s_log(LOG_DEBUG, "TLSv1.3 ciphersuites: %s", section->ciphersuites);
+        STACK_OF(SSL_CIPHER) *cipher_list, *tmp_cipher_list;
+        char *tls12_cipher_list, *tls13_cipher_list;
+
+        tmp_cipher_list=sk_SSL_CIPHER_dup(SSL_CTX_get_ciphers(section->ctx));
         if(!SSL_CTX_set_ciphersuites(section->ctx, section->ciphersuites)) {
             sslerror("SSL_CTX_set_ciphersuites");
             return 1; /* FAILED */
         }
+        cipher_list=SSL_CTX_get_ciphers(section->ctx);
+        tls12_cipher_list=compare_cipher_lists(tmp_cipher_list, cipher_list);
+        sk_SSL_CIPHER_free(tmp_cipher_list);
+        if(tls12_cipher_list) {
+            s_log(LOG_DEBUG, "TLSv1.2 and below ciphers: %s", tls12_cipher_list);
+        }
+        tls13_cipher_list=get_tls13_cipher_list(cipher_list);
+        if(tls13_cipher_list) {
+            s_log(LOG_DEBUG, "TLSv1.3 ciphersuites: %s", tls13_cipher_list);
+        }
     }
 #endif /* TLS 1.3 */
 
@@ -741,7 +759,7 @@ NOEXPORT unsigned psk_server_callback(SSL *ssl, const char *identity,
     found=psk_find(&c->opt->psk_sorted, identity);
     if(!found) {
         const char *c=identity;
-        while(*c && isprint(*c))
+        while(*c && isprint((unsigned char)*c))
             c++;
         if(*c)
             s_log(LOG_INFO, "PSK identity not found (session resumption?)");
@@ -848,7 +866,7 @@ NOEXPORT int load_pkcs12_file(SERVICE_OPTIONS *section) {
         if(i==0) { /* silence the cached attempt */
             ERR_clear_error();
         } else {
-            sslerror_queue(); /* dump the error queue */
+            sslerror(NULL); /* dump the error queue */
             s_log(LOG_ERR, "Wrong passphrase: retrying");
         }
         /* invoke the UI on subsequent calls */
@@ -926,7 +944,7 @@ NOEXPORT int load_key_file(SERVICE_OPTIONS *section) {
         if(i==0) { /* silence the cached attempt */
             ERR_clear_error();
         } else {
-            sslerror_queue(); /* dump the error queue */
+            sslerror(NULL); /* dump the error queue */
             s_log(LOG_ERR, "Wrong passphrase: retrying");
         }
         success=SSL_CTX_use_PrivateKey_file(section->ctx, section->key,
@@ -959,7 +977,7 @@ NOEXPORT int load_cert_engine(SERVICE_OPTIONS *section) {
     return 0; /* OK */
 }
 
-UI_METHOD *ui_stunnel() {
+UI_METHOD *ui_stunnel(void) {
     static UI_METHOD *ui_method=NULL;
 
     if(ui_method) /* already initialized */
@@ -990,7 +1008,7 @@ NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
             ui_stunnel(), NULL);
         if(!pkey) {
             if(i<2 && ui_retry()) { /* wrong PIN */
-                sslerror_queue(); /* dump the error queue */
+                sslerror(NULL); /* dump the error queue */
                 s_log(LOG_ERR, "Wrong PIN: retrying");
                 continue;
             }
@@ -1042,93 +1060,54 @@ NOEXPORT void set_prompt(const char *name) {
     str_free(prompt);
 }
 
-NOEXPORT int ui_retry() {
-    unsigned long err=ERR_peek_error();
-
-    switch(ERR_GET_LIB(err)) {
-    case ERR_LIB_EVP: /* 6 */
-        switch(ERR_GET_REASON(err)) {
-        case EVP_R_BAD_DECRYPT:
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_EVP error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
-    case ERR_LIB_PEM: /* 9 */
-        switch(ERR_GET_REASON(err)) {
-        case PEM_R_BAD_PASSWORD_READ:
-        case PEM_R_BAD_DECRYPT:
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_PEM error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
-    case ERR_LIB_ASN1: /* 13 */
-        return 1;
-    case ERR_LIB_PKCS12: /* 35 */
-        switch(ERR_GET_REASON(err)) {
-        case PKCS12_R_MAC_VERIFY_FAILURE:
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_PKCS12 error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
-#ifdef ERR_LIB_DSO /* 37 */
-    case ERR_LIB_DSO:
-        return 1;
+NOEXPORT int ui_retry(void) {
+    typedef struct {
+        int lib;
+        int reason;
+    } retriable;
+    const retriable retriables[]={
+        {ERR_LIB_EVP /* 6 */, EVP_R_BAD_DECRYPT},
+        {ERR_LIB_PEM /* 9 */, PEM_R_BAD_PASSWORD_READ},
+        {ERR_LIB_PEM /* 9 */, PEM_R_BAD_DECRYPT},
+        {ERR_LIB_ASN1 /* 13 */, -1},
+        {ERR_LIB_PKCS12 /* 35 */, PKCS12_R_MAC_VERIFY_FAILURE},
+#ifdef ERR_LIB_DSO
+        {ERR_LIB_DSO /* 37 */, -1},
 #endif
-    case ERR_LIB_UI: /* 40 */
-        switch(ERR_GET_REASON(err)) {
-        case UI_R_RESULT_TOO_LARGE:
-        case UI_R_RESULT_TOO_SMALL:
+        {ERR_LIB_UI /* 40 */, UI_R_RESULT_TOO_LARGE},
+        {ERR_LIB_UI /* 40 */, UI_R_RESULT_TOO_SMALL},
 #ifdef UI_R_PROCESSING_ERROR
-        case UI_R_PROCESSING_ERROR:
+        {ERR_LIB_UI /* 40 */, UI_R_PROCESSING_ERROR},
 #endif
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_UI error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
-#ifdef ERR_LIB_OSSL_STORE
-    case ERR_LIB_OSSL_STORE: /* 44 - added in OpenSSL 1.1.1 */
-        switch(ERR_GET_REASON(err)) {
-        case OSSL_STORE_R_BAD_PASSWORD_READ:
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_OSSL_STORE error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
+#ifdef ERR_LIB_OSSL_STORE /* OpenSSL 1.1.1 */
+        {ERR_LIB_OSSL_STORE /* 44 */, OSSL_STORE_R_BAD_PASSWORD_READ},
 #endif
-#ifdef ERR_LIB_PROV
-    case ERR_LIB_PROV: /* 57 - added in OpenSSL 3.0 */
-        switch(ERR_GET_REASON(err)) {
-        case PROV_R_BAD_DECRYPT:
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_PROV error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
+#ifdef ERR_LIB_PROV /* OpenSSL 3.0 */
+        {ERR_LIB_PROV /* 57 */, PROV_R_BAD_DECRYPT},
 #endif
-    case ERR_LIB_USER: /* 128 - PKCS#11 hacks */
-        switch(ERR_GET_REASON(err)) {
-        case 7UL: /* CKR_ARGUMENTS_BAD */
-        case 0xa0UL: /* CKR_PIN_INCORRECT */
-            return 1;
-        default:
-            s_log(LOG_ERR, "Unhandled ERR_LIB_USER error reason: %d",
-                ERR_GET_REASON(err));
-            return 0;
-        }
-    default:
-        s_log(LOG_ERR, "Unhandled error library: %d", ERR_GET_LIB(err));
+        /* libp11 hacks */
+        {ERR_LIB_USER /* 128 */, 7 /* CKR_ARGUMENTS_BAD */},
+        {ERR_LIB_USER /* 128 */, 0xa0 /* CKR_PIN_INCORRECT */},
+        {0, 0}
+    }, *r;
+    unsigned long err;
+    int lib, reason;
+
+    err=ERR_peek_error();
+    if(!err) { /* there is no error in the queue */
+        s_log(LOG_DEBUG, "UI giving up: No error");
         return 0;
     }
+    lib=ERR_GET_LIB(err);
+    reason=ERR_GET_REASON(err);
+    for(r=retriables; r->lib && r->reason; r++) {
+        if(r->lib==lib && (r->reason==reason || r->reason==-1)) {
+            s_log(LOG_DEBUG, "UI retrying on lib=%d reason=%d", lib, reason);
+            return 1;
+        }
+    }
+    s_log(LOG_DEBUG, "UI giving up on lib=%d reason=%d", lib, reason);
+    return 0;
 }
 
 /**************************************** session tickets */
@@ -1311,13 +1290,12 @@ NOEXPORT int sess_new_cb(SSL *ssl, SSL_SESSION *sess) {
 
     new_chain(c); /* new session -> we may have a new peer certificate chain */
 
-    if(c->opt->option.client)
-        session_cache_save(c, sess);
+    session_cache_save(c, sess);
 
     if(c->opt->option.sessiond)
         cache_new(ssl, sess);
 
-    print_session_id(sess);
+    print_session_id("Session id", sess);
 
     return 0; /* the OpenSSL's manual is really bad -> use the source here */
 }
@@ -1331,7 +1309,7 @@ NOEXPORT const unsigned char *SSL_SESSION_get_id(const SSL_SESSION *s,
 }
 #endif
 
-void print_session_id(SSL_SESSION *sess) {
+void print_session_id(const char *txt, SSL_SESSION *sess) {
     const unsigned char *session_id;
     unsigned int session_id_length;
     char session_id_txt[2*SSL_MAX_SSL_SESSION_ID_LENGTH+1];
@@ -1339,7 +1317,7 @@ void print_session_id(SSL_SESSION *sess) {
     session_id=SSL_SESSION_get_id(sess, &session_id_length);
     bin2hexstring(session_id, session_id_length,
         session_id_txt, sizeof session_id_txt);
-    s_log(LOG_INFO, "Session id: %s", session_id_txt);
+    s_log(LOG_INFO, "%s: %s", txt, session_id_txt);
 }
 
 NOEXPORT void new_chain(CLI *c) {
@@ -1390,33 +1368,56 @@ NOEXPORT void new_chain(CLI *c) {
 
 /* cache client sessions */
 NOEXPORT void session_cache_save(CLI *c, SSL_SESSION *sess) {
-    SSL_SESSION *old;
+    if(!c->opt->option.client || !sess)
+        return;
 
-#if OPENSSL_VERSION_NUMBER>=0x10100000L
-    SSL_SESSION_up_ref(sess);
-#else
-    sess=SSL_get1_session(c->ssl);
+#if OPENSSL_VERSION_NUMBER>=0x10101000L
+    if(!SSL_SESSION_is_resumable(sess))
+        return;
 #endif
 
     CRYPTO_THREAD_write_lock(stunnel_locks[LOCK_SESSION]);
-    if(c->opt->option.delayed_lookup) {
-        old=c->opt->session;
-        c->opt->session=sess;
-    } else { /* per-destination client cache */
-        if(c->opt->connect_session) {
-            old=c->opt->connect_session[c->idx];
-            c->opt->connect_session[c->idx]=sess;
-        } else {
-            s_log(LOG_ERR, "INTERNAL ERROR: Uninitialized client session cache (save)");
-            old=NULL;
-        }
-    }
-    CRYPTO_THREAD_unlock(stunnel_locks[LOCK_SESSION]);
 
-    if(old)
-        SSL_SESSION_free(old);
+    /* save per-destination client session */
+    if(c->opt->connect_session) {
+        if(c->opt->connect_session[c->idx])
+            SSL_SESSION_free(c->opt->connect_session[c->idx]);
+        c->opt->connect_session[c->idx]=SSL_SESSION_dup(sess);
+    }
+
+    /* save fallback client session */
+    if(c->opt->session)
+        SSL_SESSION_free(c->opt->session);
+    c->opt->session=SSL_SESSION_dup(sess);
+
+    CRYPTO_THREAD_unlock(stunnel_locks[LOCK_SESSION]);
 }
 
+#if OPENSSL_VERSION_NUMBER<0x10101000L
+NOEXPORT SSL_SESSION *SSL_SESSION_dup(SSL_SESSION *src) {
+    int der_len;
+    unsigned char *der_data;
+    unsigned char *tmp;
+    const unsigned char *const_tmp;
+    SSL_SESSION *dst;
+
+    der_len=i2d_SSL_SESSION(src, NULL);
+    if(der_len<=0)
+        return NULL;
+    der_data=str_alloc((size_t)der_len);
+    tmp=der_data;
+    der_len=i2d_SSL_SESSION(src, &tmp);
+    if(der_len<=0) {
+        str_free(der_data);
+        return NULL;
+    }
+    const_tmp=der_data;
+    dst=d2i_SSL_SESSION(NULL, &const_tmp, der_len);
+    str_free(der_data);
+    return dst;
+}
+#endif
+
 NOEXPORT SSL_SESSION *sess_get_cb(SSL *ssl,
 #if OPENSSL_VERSION_NUMBER>=0x10100000L
         const
@@ -1731,40 +1732,109 @@ NOEXPORT void info_callback(const SSL *ssl, int where, int ret) {
 
 /**************************************** TLS error reporting */
 
+#define MAX_ERRORS 10
+#define MAX_ERROR_LEN 256
+
 void sslerror(const char *txt) { /* OpenSSL error handler */
-    unsigned long err;
-    const char *file;
-    int line;
+    char *errors[MAX_ERRORS];
+    char *error_string;
+    int i;
 
-    err=ERR_get_error_line(&file, &line);
-    if(err) {
-        sslerror_queue();
-        sslerror_log(err, file, line, txt);
-    } else {
-        s_log(LOG_ERR, "%s: Peer suddenly disconnected", txt);
+    error_string=str_alloc(MAX_ERROR_LEN);
+    for(i=0; i<MAX_ERRORS; i++) {
+        unsigned long err=0;
+        const char *file=NULL, *func=NULL, *data=NULL;
+        int line=0, flags=0;
+
+#if OPENSSL_VERSION_NUMBER>=0x30000000L
+        err=ERR_get_error_all(&file, &line, &func, &data, &flags);
+#else
+        err=ERR_get_error_line(&file, &line);
+#endif
+        if(!err) {
+            if(txt && i==0)
+                errors[i++]=str_printf("%s: Peer suddenly disconnected", txt);
+            break;
+        }
+
+        ERR_error_string_n(err, error_string, MAX_ERROR_LEN);
+        errors[i]=str_printf("%s: %s%s%s:%d: %s%s%s",
+            txt && i==0 ? txt : "error queue",
+            func && *func ? func : "",
+            func && *func ? "@" : "",
+            file, line, error_string,
+            flags&ERR_TXT_STRING && data && *data ? ": " : "",
+            flags&ERR_TXT_STRING && data && *data ? data : "");
+    }
+    str_free(error_string);
+    ERR_clear_error();
+
+    while(i-->0) {
+        s_log(LOG_ERR, "%s", errors[i]);
+        str_free(errors[i]);
     }
 }
 
-NOEXPORT void sslerror_queue(void) { /* recursive dump of the error queue */
-    unsigned long err;
-    const char *file;
-    int line;
+/**************************************** ciphersuites */
+#ifndef OPENSSL_NO_TLS1_3
+NOEXPORT char *compare_cipher_lists(STACK_OF(SSL_CIPHER) *list1, STACK_OF(SSL_CIPHER) *list2) {
+    char *result=NULL;
+    size_t result_len=0;
+    int i;
 
-    err=ERR_get_error_line(&file, &line);
-    if(err) {
-        sslerror_queue();
-        sslerror_log(err, file, line, "error queue");
+    for(i=0; i<sk_SSL_CIPHER_num(list2); i++) {
+        const SSL_CIPHER *cipher2=sk_SSL_CIPHER_value(list2, i);
+        const char *cipher2_name=SSL_CIPHER_get_name(cipher2);
+        int found=0;
+
+        for(int j=0; j<sk_SSL_CIPHER_num(list1); j++) {
+            const SSL_CIPHER *cipher1=sk_SSL_CIPHER_value(list1, j);
+            const char *cipher1_name=SSL_CIPHER_get_name(cipher1);
+
+            if(!strcmp(cipher2_name, cipher1_name)) {
+                found=1;
+                break;
+            }
+        }
+        if(!found) {
+            size_t name_len=strlen(cipher2_name);
+
+            result=realloc(result, result_len + name_len + 2); /* +2 for ':' and '\0' */
+            if(result_len == 0) {
+                strcpy(result, cipher2_name);
+            } else {
+                strcat(result, ":");
+                strcat(result, cipher2_name);
+            }
+            result_len+=name_len + 1; /* +1 for ':' */
+        }
     }
+    return result;
 }
 
-NOEXPORT void sslerror_log(unsigned long err,
-        const char *file, int line, const char *txt) {
-    char *str;
+NOEXPORT char *get_tls13_cipher_list(STACK_OF(SSL_CIPHER) *list) {
+    char *result=NULL;
+    size_t result_len=0;
+    int i;
 
-    str=str_alloc(256);
-    ERR_error_string_n(err, str, 256);
-    s_log(LOG_ERR, "%s: %s:%d: %s", txt, file, line, str);
-    str_free(str);
+    for(i=0; i<sk_SSL_CIPHER_num(list); i++) {
+        const SSL_CIPHER *cipher=sk_SSL_CIPHER_value(list, i);
+
+        if(!strcmp(SSL_CIPHER_get_version(cipher), "TLSv1.3")) {
+            const char *cipher_name=SSL_CIPHER_get_name(cipher);
+            size_t name_len=strlen(cipher_name);
+
+            result=realloc(result, result_len + name_len + 2); /* +2 for ':' and '\0' */
+            if(result_len == 0) {
+                strcpy(result, cipher_name);
+            } else {
+                strcat(result, ":");
+                strcat(result, cipher_name);
+            }
+            result_len+=name_len + 1; /* +1 for ':' */
+        }
+    }
+    return result;
 }
-
+#endif /* TLS 1.3 */
 /* end of ctx.c */
