@@ -111,7 +111,7 @@ NOEXPORT void threadid_func(CRYPTO_THREADID *tid) {
 }
 #endif
 
-NOEXPORT void thread_id_init() {
+NOEXPORT void thread_id_init(void) {
 #if OPENSSL_VERSION_NUMBER>=0x10000000L && OPENSSL_VERSION_NUMBER<0x10100000L
     CRYPTO_THREADID_set_callback(threadid_func);
 #endif
@@ -389,7 +389,7 @@ int CRYPTO_atomic_add(int *val, int amount, int *ret, CRYPTO_RWLOCK *lock) {
 
 #endif /* OPENSSL_VERSION_NUMBER<0x10100004L */
 
-NOEXPORT void locking_init() {
+NOEXPORT void locking_init(void) {
     size_t i;
 #if defined(USE_OS_THREADS) && OPENSSL_VERSION_NUMBER<0x10100004L
     size_t num;
